import 'package:flutter/material.dart';
import 'package:flutter/scheduler.dart';
import 'package:get/get.dart';
import 'package:wowit/Controller/baseController.dart';
import 'package:wowit/Controller/connectController.dart';
import 'package:wowit/Widget/loadingWidget.dart';
import 'package:wowit/Widget/screenMessage.dart';
import 'package:wowit/common.dart';

abstract class BaseWidget<T extends BaseController> extends GetWidget<T> {
  final String tag;
  final T _baseController;
  final MyConnectivity _connectivity = MyConnectivity.instance;

  BaseWidget(
    T baseController, {
    bool isComponent = true,
    this.tag,
  }) : _baseController = baseController {
    _baseController.isComponent = isComponent;
    Get.lazyPut<T>(() => _baseController, tag: tag);
     _connectivity.initialise(_baseController);
  }

  T getController<T>({String tag}) {
    return Get.find<T>(tag: tag);
  }

  @override
  Widget build(BuildContext context) {
    return this._getWidget(context);
  }

  void initStateGetX() async {
    printInfo(info: this.toString() + " initStateGetX");
  }

  Widget _getWidget(BuildContext context) {
    return GetBuilder<T>(
      init: _baseController,
      tag: tag,
      initState: (_) async {
        if (Common.supperAdmin && !_baseController.isComponent)
          SchedulerBinding.instance.addPostFrameCallback((_) {
            _baseController.messageIdBackUp = Common.messageIdBackUp;
          });
        this.initStateGetX();
      },
      builder: (baseController) {
        printInfo(
          info: this.toString() +
              "  GetWidget start with Controller: " +
              baseController.toString(),
        );
        var cachedData = baseController.getData();
        if (cachedData == null || cachedData.length == 0) {
          return FutureBuilder(
            future: baseController.loadData(),
            builder: (
              context,
              AsyncSnapshot<Map<String, List<Map<String, Object>>>> snapshot,
            ) {
              printInfo(info: this.toString() + " GetWidget rebuild new data");
              if (snapshot.connectionState == ConnectionState.waiting) {
                printInfo(
                    info: this.toString() +
                        " ConnectionState.waiting return getSkeletonsView");
                return this.getSkeletonView(context, baseController);
              } else if (snapshot.hasError) {
                printInfo(
                    info: this.toString() +
                        " Has error return getErrorView message: " +
                        snapshot.error.toString());
                return this._errorViewAndNoInterNet(snapshot, baseController);
              } else if (snapshot.data != null && snapshot.data.length == 0) {
                printInfo(
                    info: this.toString() +
                        " Data is empty data.length == 0 return getEmptyView");
                return this._emptyViewAndNoInterNet(baseController);
              }
              baseController.setData(snapshot.data);
              printInfo(
                  info: this.toString() +
                      " Done process return getMainView with Controller: " +
                      baseController.toString());
              if (Common.supperAdmin && !baseController.isComponent) {
                return _widgetShowMessageID(
                  context,
                  this.getMainView(context, snapshot.data, baseController),
                  baseController,
                );
              }
              return this.getMainView(context, snapshot.data, baseController);
            },
          );
        } else {
          printInfo(info: this.toString() + " GetWidget rebuild old data");
          return this.getMainView(context, cachedData, baseController);
        }
      },
    );
  }

  Widget getWaitingView(BuildContext context) {
    printInfo(info: "getWaitingView: " + context.toString());
    return Common.loadingView ?? LoadingScreen();
  }

  Widget getSkeletonView(BuildContext context, BaseController baseController) {
    printInfo(
        info: "getSkeletonView: " +
            context.toString() +
            " _isComponent: " +
            baseController.isComponent.toString());
    //show waiting view when is not component
    if (!baseController.isComponent) {
      return getWaitingView(context);
    }
    return Padding(
      padding: const EdgeInsets.only(left: 10),
      child: Column(
        mainAxisAlignment: MainAxisAlignment.start,
        crossAxisAlignment: CrossAxisAlignment.start,
        children: [
          Row(
            children: [
              Expanded(
                flex: 6,
                child: Padding(
                  padding: const EdgeInsets.all(8.0),
                  child: Container(
                    height: 150,
                    decoration: BoxDecoration(
                      borderRadius: BorderRadius.circular(10),
                      color: Colors.grey[100],
                    ),
                  ),
                ),
              ),
              Expanded(
                flex: 3,
                child: Column(
                  children: [
                    Padding(
                      padding: const EdgeInsets.all(8.0),
                      child: Container(
                        height: 67,
                        decoration: BoxDecoration(
                          borderRadius: BorderRadius.circular(10),
                          color: Colors.grey[100],
                        ),
                      ),
                    ),
                    Padding(
                      padding: const EdgeInsets.all(8.0),
                      child: Container(
                        height: 67,
                        decoration: BoxDecoration(
                          borderRadius: BorderRadius.circular(10),
                          color: Colors.grey[100],
                        ),
                      ),
                    ),
                  ],
                ),
              )
            ],
          ),
          Padding(
            padding: const EdgeInsets.only(
              top: 8,
              right: 8,
              left: 8,
              bottom: 8,
            ),
            child: Container(
              height: 20,
              width: MediaQuery.of(context).size.width,
              decoration: BoxDecoration(
                borderRadius: BorderRadius.circular(10),
                color: Colors.grey[100],
              ),
            ),
          ),
          Padding(
            padding:
                const EdgeInsets.only(top: 8, right: 20, left: 8, bottom: 8),
            child: Container(
              height: 20,
              width: MediaQuery.of(context).size.width / 1.5,
              decoration: BoxDecoration(
                borderRadius: BorderRadius.circular(10),
                color: Colors.grey[100],
              ),
            ),
          )
        ],
      ),
    );
  }

  Widget _errorViewAndNoInterNet(
      AsyncSnapshot<Map<String, List<Map<String, Object>>>> snapshot,
      BaseController baseController){
    if (baseController.isOnline) {
      baseController.loadData();
      return getErrorView(snapshot,baseController);
    } else{
      return noInternet(); 
    }
  }

  Widget getErrorView(
      AsyncSnapshot<Map<String, List<Map<String, Object>>>> snapshot,
      BaseController baseController) {
    printInfo(info: "getErrorView: " + snapshot.error.toString());
    return Center(
        child: Image.asset(
          "assets/images/404-error.png",
          height: 100,
          width: 100,
        ),
      );
  }

  Widget _emptyViewAndNoInterNet(BaseController baseController){
    if (baseController.isOnline) {
      baseController.loadData();
      return getEmptyView(baseController);
    } else {
      return noInternet();
    }
  }

  Widget getEmptyView(BaseController baseController) {
    return Center(
        child: Image.asset(
          "assets/images/emptyFolder.png",
          height: 100,
          width: 100,
        ),
      );
  }

  Widget noInternet() {
    print('no internet');
    return Common.widgetNotificationInternet ??
        Center(
          child: Image.asset(
            "assets/images/404-error.png",
            height: 100,
            width: 100,
          ),
        );
  }

  Widget _widgetShowMessageID(
    BuildContext context,
    Widget mainView,
    BaseController baseController,
  ) {
    return Stack(
      children: [
        mainView,
        Positioned(
          right: 10,
          bottom: 50,
          child: Container(
            child: Material(
              borderRadius: BorderRadius.circular(100),
              child: InkWell(
                onTap: () {
                  // Common.showKeyMessage = !Common.showKeyMessage;
                  // baseController.update();
                  showAlertDialog(context, baseController);
                },
                child: Container(
                  width: 50,
                  height: 50,
                  padding: const EdgeInsets.all(8),
                  decoration: BoxDecoration(
                    color: Colors.blue[50],
                    shape: BoxShape.circle,
                  ),
                  child: Center(
                    child: !Common.showKeyMessage
                        ? Icon(Icons.visibility_outlined)
                        : Icon(
                            Icons.visibility_off_outlined,
                          ),
                  ),
                ),
              ),
            ),
          ),
        )
      ],
    );
  }

  Widget getMainView(
    BuildContext context,
    Map<String, List<Map<String, Object>>> data,
    BaseController controller,
  );

  showAlertDialog(BuildContext context, BaseController baseController) {
    // set up the button
    Widget dsMessageButton = TextButton(
      child: Text("Danh sách message"),
      onPressed: () {
        Get.back();
        Get.to(
          () => ScreenMessage(
            data: baseController.messageIdBackUp,
          ),
        );
      },
    );
    Widget displayMessageIdButton = TextButton(
      child: Text(Common.notGetMessage
          ? "Hiển thị bình thường"
          : "Hiển thị message ID"),
      onPressed: () {
        Common.notGetMessage = !Common.notGetMessage;
        baseController.update();
        Get.back();
      },
    );
    // set up the AlertDialog
    AlertDialog alert = AlertDialog(
      title: Text("Supper Admin"),
      actions: [
        dsMessageButton,
        displayMessageIdButton,
      ],
    );

    // show the dialog
    showDialog(
      context: context,
      builder: (BuildContext context) {
        return alert;
      },
    );
  }
}
